#!/bin/bash
#
# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2020. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************
#

if [ "${LIB_ERROR_SOURCED}" = "" ] ; then
LIB_ERROR_SOURCED="true"

declare -r MESSAGE_CATEGORY=2
declare -r MSG_NUM_PROC_UNSUPPORTED="1 Specifying the number of processes with \${next_arg} has been deprecated and is no longer supported."
declare -r MSG_RANKGEN_PARAM_UNSUPPORTED="2 \${next_arg} has been deprecated and is no longer supported."
declare -r MSG_HW_SPEC_ERROR="3 An error was encountered trying to determine local hardware specs, defaulting to use a single mpi instance."
declare -r MSG_RANK_ARG_FILE="5 Rankgen args were provided, with a rankfile."
declare -r MSG_RANK_FILE_HOST="6 A rankfile and a host list cannot be used at the same time."
declare -r MSG_RANK_FILE_ERROR="10 An error was encountered trying to generate the rankfile."
declare -r MSG_SW_ERROR="11 An error was encountered verifying the software exists on \${SPLIT_HOSTS[i]}.\\\n See \${WORKING_DIR}/\${SPLIT_HOSTS[i]}_software.err for detailed error message. Continuing to test other host software."
declare -r MSG_HOST_CFG="12 An error was encountered verifying the host config on \${SPLIT_HOSTS[i]}.\\\nContinuing to test host configurations."
declare -r MSG_DIFF_HOST_CFG="13 All machine configurations are not the same. Please verify that the correct hosts are being used."
declare -r MSG_NO_RANKFILE="14 Cannot find rankfile \${RANK_FILE}."
declare -r MSG_NO_MPIRUN="15 Cannot find 'mpirun' executable. Do you need to run an activate script?"
declare -r MSG_MPI_OPS="16 An error was encountered trying to use the specified MPI options."
declare -r MSG_NO_MPITOOL="17 Not performing connection tests. Cannot find 'mpitool' executable. This could be because you are using a version of mpi that does not ship with mpitool."
declare -r MSG_SSH_ERROR="18 An error was encountered trying to ssh to \${HOSTS[i]}. See \${WORKING_DIR}/sshTest\${HOSTS[i]}.out for detailed error message.\\\nContinuing to test host connectivity."
declare -r MSG_NO_AUTO_RANKFILE="19 \${RANKFILE} does not appear to be an auto-generated rankfile."
declare -r MSG_DIFF_CORES="21 Hosts have different numbers of cores. The minimum core count is being used. This may lead to sub-optimal performance."
declare -r MSG_NO_EXECUTABLE="22 The program \${EXECUTABLE} does not exist or is not executable on \${SPLIT_HOSTS[i]}. See \${WORKING_DIR}/\${SPLIT_HOSTS[i]}_executable.err for detailed error message."
declare -r MSG_HOSTFILE_AND_HOSTLIST="23 A host file and host list cannot be used at the same time."
declare -r MSG_HOSTFILE_AND_RANKFILE="24 A host file and rankfile cannot be used at the same time."
declare -r MSG_NO_HOSTFILE="25 Specified hostfile \${HOST_FILE} does not exist, or is not accessible."
declare -r MSG_HOSTFILE_COMMA_SPACE="26 Hostfile \${HOST_FILE} contains lines with uncommented commas and/or spaces. Only one host name is allowed per line."
declare -r MSG_INVALID_RANKGEN_ARG="27 The value \${ARG_VALUE} is not a valid option for \${next_arg}. All rankgen/topology arguments must be positive integers."
declare -r MSG_INFINIBAND_NOT_SETUP="28 InfiniBand is not properly configured on host \'\${HOST}\'. Use \'--mpiarg -pami_noib\' flag for single host runs or \'--tcp\' for multi-host TCP communication."
declare -r MSG_MPI_CONNECT_ERROR="29 MPI conncetion test failed. See \${WORKING_DIR}/connectTest\${HOSTS[i]}.out for detailed error message."

##############################################################################
# Print error message to stderr.
# Globals:
#   ${MESSAGE_CATEGORY}
# Arguments:
#   Severity
#   Message variable ${MSG_}
# Returns:
#   None
##############################################################################
print_error()
{
  local severity=$1
  shift
  local raw_message="$@"
  local code=${raw_message%% *}
  local message=${raw_message#* }
  eval >&2 echo -e "[$severity SNAPML-$MESSAGE_CATEGORY-$code] $message"
}

fi
