#!/bin/bash
#*******************************************************************************
# Licensed Materials - Property of IBM
# 5725-Y38
# @ Copyright IBM Corp. 2018, 2019. All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#*******************************************************************************
# Notes:
# - echo to stdout to show output in Conductor worker log file


SCRIPT=$(readlink -f "$0")
SCRIPT_NAME=$(basename "$SCRIPT")
WORK_HOME=$(dirname "$SCRIPT")
WORK_HOME=$(readlink -f "$WORK_HOME")

source ${WORK_HOME}/common_wrapper.sh
trace_env_com
activate_conda_com

CONDA_BIN=`which conda`
CONDA_PATH=`dirname $CONDA_BIN`

#
# change these variables if needed
#
moduleName=snapml_wrapper
workerProp=worker${WORKER_INDEX}.prop
mpiRunWrapper=snapmlrun_wrapper.sh

#
# expect these variables to be exported by the caller
#
echo "SNAPML_WORKER_COUNT="${WORKER_COUNT} > ${workerProp}
echo "SNAPML_WORKER_INDEX="${WORKER_INDEX} >> ${workerProp}
echo "SNAPML_HOSTNAME="${WORKER_HOSTNAME} >> ${workerProp}
echo "SNAPML_CUDA_VISIBLE_DEVICES="${CUDA_VISIBLE_DEVICES} >> ${workerProp}

#
# If this is worker 0, then collect info from all workers. If not then exit.
# worker 0 will generate  file and run snaprun
#
if [ ${WORKER_INDEX} -ne 0 ]
then
   echo "[$moduleName]: worker ${WORKER_INDEX} exits. Only worker 0 runs snaprun"
   exit 0
fi

#
# This is worker 0, collect info from all workers, generate:
# - RANK_HOSTS string to contain all distinct hosts
# - RANK_HOST_COUNT is number of RANK_HOSTS
echo "[$moduleName]: worker count=${WORKER_COUNT}"
RANK_HOSTS=""
RANK_HOST_COUNT="0"
i="0"
while [ $i -lt ${WORKER_COUNT} ]
do
   file=./worker${i}.prop
   # wait for file for 600 seconds
   j="0"
   while [ ! -f "$file" -a $j -lt 60 ]
   do
      echo "[$moduleName]: wait for file ${file}."
      sleep 10
      j=$[$j+1]
   done

   if [ ! -f "$file" ]
   then
      echo "[$moduleName]: cannot find file ${file}. Exit."
      exit 1
   fi

   echo "[$moduleName]: reading ${file}"
   . ${file}

   echo "[$moduleName]: HOSTNAME=${SNAPML_HOSTNAME}, RANK_HOSTS=$RANK_HOSTS"
   if [[ $RANK_HOSTS != *"${SNAPML_HOSTNAME}"* ]]
   then
      echo "[$moduleName]: add ${SNAPML_HOSTNAME}"
      # add to the list
      if [[ $RANK_HOSTS != "" ]]
      then
         RANK_HOSTS="$RANK_HOSTS,"
      fi
      RANK_HOSTS="$RANK_HOSTS${SNAPML_HOSTNAME}"
      RANK_HOST_COUNT=$[$RANK_HOST_COUNT+1]

   fi

   i=$[$i+1]
done
echo "[$moduleName]: RANK_HOST_COUNT=${RANK_HOST_COUNT}"
echo "[$moduleName]: RANK_HOSTS=${RANK_HOSTS}"


#
# generate a mpi_wrapper.sh to control CUDA_VISIBLE_DEVICES on each host for this snaprun so CUDA_VISIBLE_DEVICES may
# be set differently on each node for this snaprun
#
echo "#!/bin/bash" > $mpiRunWrapper
echo "# This file is auto generated" >> $mpiRunWrapper
echo "unset CUDA_VISIBLE_DEVICES" >> $mpiRunWrapper

#
# Extract all python arguments from input arguments. Also replace _QUOTE_ with quotes
#
echo "[$moduleName]: Getting arguments "
MPI_ARGS=""
CMD_ARGS=""

for var in "$@"
do
   echo "[$moduleName]: ${var}"
   if [[ ${var} == "python" ]]
   then
      CMD_ARGS="python"
   else
      if [[ ${var} == *"_QUOTE_"* ]]
      then
         v=`echo ${var} | sed 's/_QUOTE_/\"/g'`
      else
         v=${var}
      fi

      if [[ ${CMD_ARGS} == "" ]]
      then
         MPI_ARGS="${MPI_ARGS} $v"
      else
         CMD_ARGS="${CMD_ARGS} $v"
      fi

   fi
done
# make sure to be in the right directory
echo "export DLI_SNAPML_MPI=1" >> $mpiRunWrapper
echo "if ! echo \$PATH | /bin/grep -q \"$CONDA_PATH\"; then export PATH=$CONDA_PATH:\$PATH; fi" >> $mpiRunWrapper
echo "conda activate ${CONDA_DEFAULT_ENV}" >> $mpiRunWrapper
echo "cd `pwd`" >> $mpiRunWrapper
echo "${CMD_ARGS}" >> $mpiRunWrapper
chmod 755 $mpiRunWrapper
echo "[$moduleName]: CMD_ARGS=${CMD_ARGS}"

#
# All is ready, call mpirun
#
echo "[$moduleName]: snaprun -H ${RANK_HOSTS}  $MPI_ARGS `pwd`/$mpiRunWrapper"
snaprun -v -H ${RANK_HOSTS} $MPI_ARGS `pwd`/$mpiRunWrapper

