# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019, 2020. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

# Sample program to use kmeans from cuML in pai4sk

import sys
if sys.version_info.major != 3:
    raise ValueError("cuML in pai4sk is supported only with python 3.6.")

from pai4sk.cluster import KMeans as pai4skKMeans
from sklearn.cluster import KMeans as skKMeans
from pai4sk.metrics.cluster import v_measure_score
import cudf
import numpy as np
import pandas as pd
import time

import argparse

CLI=argparse.ArgumentParser()
CLI.add_argument(
    "--nrows",
    type=int,
    default=1000000
)
CLI.add_argument(
    "--ncols",
    type=int,
    default=100
)
CLI.add_argument(
    "--nclusters",
    type=int,
    default=2
)
CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
args = CLI.parse_args()

from sklearn.datasets import make_blobs
# Create a sample dataset with n clusters
X, y = make_blobs(n_samples=args.nrows, n_features=args.ncols, centers=args.nclusters)

dict1 = {}
for i in range(0, args.ncols):
    dict1["a"+str(i)] = X[:,i]

pdf = pd.DataFrame(dict1)

cdf = cudf.DataFrame.from_pandas(pdf)

pai4sk_km = pai4skKMeans(n_clusters=args.nclusters)
t0 = time.time()
pai4sk_km.fit(cdf)
print("[pai4sk] Training time (s):  {0:.2f}".format(time.time()-t0))
#print(pai4sk_km.cluster_centers_)
#print(pai4sk_km.labels_)

if args.skip_sklearn:
    print('Execution using scikit-learn is skipped.')
    exit()

sk_km = skKMeans(n_clusters=args.nclusters)
t0 = time.time()
sk_km.fit(pdf)
print("[sklearn] Training time (s):  {0:.2f}".format(time.time()-t0))
#print(pd.DataFrame(sk_km.cluster_centers_))
#print(sk_km.labels_)

print("How perfect is the kmeans labels assignment (up to a permutation) when\n" \
        "compared to scikit-learn ? Score 1.0 means 100% perfect. Score = {0:.1f}\n".
        format(v_measure_score(sk_km.labels_, pai4sk_km.labels_.to_array())))

