# Abalone Example

In this example we will train Decision Tree and Random Forest models on the abalone dataset, using both `scikit-learn` and `snap-ml-local`. 

## Download the Data

Download and decompress the data from the LIBSVM repository:
```
mkdir data
cd data
wget https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/regression/abalone_scale
mv abalone_scale abalone
cd ../
```

## Data Pre-processing
The data is in SvmLight format. Here we perform some pre-processing. 

The entire pre-processing can be run by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-abalone/preprocess-abalone.py`.

Command to pre-process the data available in specific directory:
`python $CONDA_PREFIX/pai4sk/local-examples/example-abalone/preprocess-abalone.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.


## Training and Evaluating a Decision Tree Regressor Model

One can run the entire example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-abalone/example-abalone-decision-tree.py`

Command to run the example using the pre-processed data in specific directory.
`python $CONDA_PREFIX/pai4sk/local-examples/example-abalone/example-abalone-decision-tree.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

One should see the following output on P9+V100 system:
```
Data load time (s): 0.01
[snap] Training time (s): 12.25
[snap] Mean squared error score: 9.8708
[snap.histogram] Training time (s): 1.93
[snap.histogram] Mean squared error score: 9.4287
[sklearn] Training time (s): 14.63
[sklearn] Mean squared error score: 9.8794
```

For the decision tree and random forest models, `snap-ml-local` provides a feature called `histograms`: when enabled, 
the values of each feature are grouped into a user-defined number of bins; during training, the splits are evaluated at the bin values instead
of the individual feature values. This can significantly accelerate the training of the model. To enable `histograms`, the user needs to define two
parameters at model instantiation time: `use_histograms` and `hist_nbins`. An example of how to use these parameters
can be found in `$CONDA_PREFIX/pai4sk/snap-ml-local/local-examples/example-abalone/example-abalone-decision-tree.py`. The `sklearn` library 
does not offer (as of version 0.22) a decision tree / random forest API with `histograms` functionality. Nevertheless, in this example,
we compare the `snap-ml-local` `histograms`-based decision tree with the `sklearn` exact decision tree algorithm and show the benefits of 
using `histograms` both in terms of training time and generalization performance.


## Training and Evaluating a Random Forest Regressor Model

One can run the entire example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-abalone/example-abalone-random-forest.py`

Command to run the example using the pre-processed data in specific directory.
`python $CONDA_PREFIX/pai4sk/local-examples/example-abalone/example-abalone-random-forest.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

One should see the following output on P9+V100 system:
```
Data load time (s):  0.01
[snap] Training time (s):  4.24
[snap] Mean squared error score:   4.9696
[snap.histogram] Training time (s):  2.00
[snap.histogram] Mean squared error score:   4.8150
[sklearn] Training time (s):  5.31
[sklearn] Mean squared error score:   4.9931
```

&copy; Copyright IBM Corporation 2018, 2019
