# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019, 2020. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from scipy import sparse

# Load the data
import time
import numpy as np
import argparse

defaultPath = "."
CLI=argparse.ArgumentParser()


CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--data_path",
    type=str,
    default=defaultPath
)
CLI.add_argument(
    "--num_threads",
    type=int,
    default=8
)

args = CLI.parse_args()


t0 = time.time()
X_train = np.load(args.data_path + "/data/HIGGS.X_train.npy")
X_test  = np.load(args.data_path + "/data/HIGGS.X_test.npy")
y_train = np.load(args.data_path + "/data/HIGGS.y_train.npy")
y_test  = np.load(args.data_path + "/data/HIGGS.y_test.npy")
print("Data load time (s):  {0:.2f}".format(time.time()-t0))

max_depth = 16

for use_gpu in [False, True]:

    # Import snap DecisionTreeClassifier from pai4sk module directly
    from pai4sk import DecisionTreeClassifier as SnapTree
    dt = SnapTree(random_state=0, verbose=False, max_depth=max_depth, use_histograms=True, use_gpu=use_gpu, n_threads=args.num_threads)

    # Training time
    t0 = time.time()
    dt.fit(X_train, y_train)
    t_fit = time.time()-t0
    print("[snap.%s] Training time (s):  %.2f" % ('gpu' if use_gpu else 'cpu', t_fit))

    # Inference
    pred_test = dt.predict(X_test)

    from sklearn.metrics import accuracy_score
    # Evaluate accuracy_score on test set
    acc_snap = accuracy_score(y_test, pred_test)
    print("[snap.%s] Accuracy score:   %.4f" % ('gpu' if use_gpu else 'cpu', acc_snap))


if not args.skip_sklearn:
    # Import sklearn's DecisionTreeClassifier from pai4sk module directly
    from pai4sk.tree import DecisionTreeClassifier as SklearnTree

    dt = SklearnTree(random_state=0, max_depth=max_depth)

    # Training time
    t0 = time.time()
    dt.fit(X_train, y_train)
    print("[sklearn] Training time (s):  {0:.2f}".format(time.time()-t0))

    # Inference
    pred_test = dt.predict(X_test)

    # Evaluate accuracy_score on test set
    acc_sklearn = accuracy_score(y_test, pred_test)
    print("[sklearn] Accuracy score:   {0:.4f}".format(acc_sklearn))
