# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2020. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from scipy import sparse

# Load the data
import time
import numpy as np
import argparse

from sklearn.metrics import accuracy_score

defaultPath = "."
CLI=argparse.ArgumentParser()

CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--data_path",
    type=str,
    default=defaultPath
)
CLI.add_argument(
    "--num_threads",
    type=int,
    default=8
)
CLI.add_argument(
    "--num_gpus",
    type=int,
    default=2
)

args = CLI.parse_args()

t0 = time.time()
X_train = np.load(args.data_path + "/data/SUSY.X_train.npy")
X_test  = np.load(args.data_path + "/data/SUSY.X_test.npy")
y_train = np.load(args.data_path + "/data/SUSY.y_train.npy")
y_test  = np.load(args.data_path + "/data/SUSY.y_test.npy")
print("Data load time (s):  {0:.2f}".format(time.time()-t0))

random_state = 42
max_depth = 10
n_estimators = 320

# Import snap RandomForestClassifier from pai4sk module directly
from pai4sk import RandomForestClassifier as SnapForest

use_gpu_range = [False, True] if args.num_gpus > 0 else [False]
gpu_ids = range(args.num_gpus)

for use_gpu in use_gpu_range:

    rf = SnapForest(random_state=random_state, max_depth=max_depth, n_estimators=n_estimators, n_jobs=args.num_threads, max_features='sqrt',
                        use_histograms=True, use_gpu=use_gpu, gpu_ids=gpu_ids)

    # Training time
    t0 = time.time()
    rf.fit(X_train, y_train)
    t_fit = time.time()-t0
    print("[snap.%s] Training time (s):  %.2f" % ('gpu' if use_gpu else 'cpu', t_fit))

    # Inference
    pred_test = rf.predict(X_test)
    acc_snap = accuracy_score(y_test, pred_test)
    print("[snap.%s] Accuracy score:   %.4f" % ('gpu' if use_gpu else 'cpu', acc_snap))


if not args.skip_sklearn:
    # Import sklearn's RandomForestClassifier from pai4sk module directly
    from pai4sk.ensemble import RandomForestClassifier as SklearnForest

    rf = SklearnForest(random_state=random_state, max_depth=max_depth, n_estimators=n_estimators, n_jobs=args.num_threads, max_features='sqrt')

    # Training time
    t0 = time.time()
    rf.fit(X_train, y_train)
    print("[sklearn] Training time (s):  {0:.2f}".format(time.time()-t0))

    # Inference
    pred_test = rf.predict(X_test)

    # Evaluate accuracy_score on test set
    acc_sklearn = accuracy_score(y_test, pred_test)
    print("[sklearn] Accuracy score:   {0:.4f}".format(acc_sklearn))


